#ifdef SCCS
static char sccsid[]="@(#)browser.c	1.4 Stellar 89/03/22";
#endif
/*
			Copyright 1989 by
			Stellar Computer Inc.
			All Rights Reserved
	
	This software comprises unpublished confidential information of
	Stellar Computer Inc. and may not be used, copied or made
	available to anyone, except in accordance with the license
	under which it is furnished.
*/
/*
 * LUI Browser
 */

#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <dirent.h>

#define LUI_BROWSER_DECLARE
#include "lui.h"

static char
  *PathTail(),
  *PathTailUp();

/******************************************************************************/
 LUI_BROWSER *
LUI_BrowserOpen(name, parent, x, y, width, height, 
		func, page_color, background_color, title_pixmap_file)
    char *name;
    Window parent;
    int x, y, width, height;
    LUI_FNCP func;
    unsigned long page_color, background_color;
    char *title_pixmap_file;
{
    Window big_window, window, browser_window;
    LUI_LABEL *up_label, *down_label, *dir_label, *slidebar_label;
    LUI_BROWSER *browser;
    static int been_here = 0;
    int w, bw, bh, LUI_BrowserDispatch();

    *LUI_BrowserFilename = '\0';
    
    if (!been_here) {
	been_here = 1;

	/* Allocate the browser */
	if (!(browser = (LUI_BROWSER *)malloc(sizeof(LUI_BROWSER)))) {
	    printf("LUI_Browser: can't allocate browser\n");
	    return(NULL);
	}

	/* Read the title pixmap if there is one */
	if (title_pixmap_file) {
	    if (!(browser->title_pixmap = 
		  LUI_ReadPixmap(title_pixmap_file,
				 0, 0, &(browser->title_width), &(browser->title_height)))) {
		fprintf(stderr, "LUI_BrowserOpen(): can't read title pixmap %s\n", 
			title_pixmap_file);
	    }
	}
	
	/* Create the big window for browser and buttons */
	big_window = LUI_CreateWindowAt(parent, x, y, width, height);
	XSetWindowBackground(LUI_Display, big_window, page_color);
	browser->big_window = big_window;
	browser->big_x = x;
	browser->big_y = y;
	browser->big_width = width;
	browser->big_height = height;

	/* Create the directory label */
	dir_label = LUI_LabelCreate("Browser_DIR", "** NOT SET YET **",
				    big_window, 20, 40, width - 36, 22,
				    0, NULL, 1, 1);
	XSetWindowBackground(LUI_Display, dir_label->window, page_color);
	LUI_LabelDropShadow("Browser_DIR", 1);

	/* Create the browser buttons */
	w = width/2 - 20;
	LUI_ButtonPadOpen("LUI_BrowserMain", big_window, 20, height - 30, 0, page_color);
          LUI_ButtonCreate("Change Dir", LUI_BUTTON_ONESHOT, 0, 0, w, LUI_BrowserDispatch);
          LUI_ButtonCreate("Close ",     LUI_BUTTON_ONESHOT, 1, 0, w, LUI_BrowserDispatch);
	LUI_ButtonPadClose("LUI_BrowserMain");
	LUI_ButtonPadVisible("LUI_BrowserMain", 1);

	/* Create the browser main window */
	x = 20;
	y = 70;
	width -= 40;
	height -= 110;

	window = LUI_CreateWindowAt(big_window, x, y, width, height);
	XSetWindowBackground(LUI_Display, window, page_color);

	/* Create the browser scrolling window */
	bw = width;
	bh = height;
	browser_window = LUI_CreateWindowAt(window, 0, 0, bw, bh);
	XSetWindowBackground(LUI_Display, browser_window, page_color);

	w = width;

	/* Create the labels */
	up_label = LUI_LabelCreate("Browser_UP", " ", window, 
				   width - 20, 0, 20, 20,
				   0, LUI_BrowserLabelProcess, 0, 1);
	up_label->draw_func = LUI_BrowserLabelDrawArrow;
	LUI_Cursor(up_label->window, LUI_UP_DOWN_CURSOR);

	down_label = LUI_LabelCreate("Browser_DOWN", " ", window, 
				     width - 20, height - 20, 20, 20,
				     0, LUI_BrowserLabelProcess, 0, 1);
	down_label->draw_func = LUI_BrowserLabelDrawArrow;
	LUI_Cursor(down_label->window, LUI_UP_DOWN_CURSOR);

	slidebar_label = LUI_LabelCreate("Browser_SLIDE", " ", window, 
				     width - 20, 20, 
				     20, height - 40,
				     0, LUI_BrowserLabelProcess, 0, 1);
	LUI_Cursor(slidebar_label->window, LUI_UP_DOWN_CURSOR);

	/* Add Browser events */
	LUI_EventAdd(big_window, ExposureMask, LUI_BrowserWindowProcess);
	LUI_EventAdd(window, ExposureMask, LUI_BrowserWindowProcess);
	LUI_EventAdd(browser_window,
		     KeyPressMask    | KeyReleaseMask  |
		     EnterWindowMask | LeaveWindowMask | PointerMotionMask |
		     ExposureMask    | ButtonPressMask | ButtonReleaseMask,
		     LUI_BrowserProcess);
	
	/* Setup browser variables */
	browser->x              = x;
	browser->y              = y;
	browser->width          = width;
	browser->height         = height;
	browser->browser_width  = bw;
	browser->browser_height = bh;
	browser->big_window     = big_window;
	browser->window         = window;
	browser->visible        = 1;
	browser->func           = func;
	browser->font           = LUI_Font;
	browser->font_yoff      = LUI_Font_yoff;
	browser->data           = NULL;
	browser->file_ind       = 0;
	browser->line_space     = LUI_Font_yoff + LUI_BROWSER_LINE_SPACE;
	browser->browser_window = browser_window;
	browser->up_label       = up_label;
	browser->down_label     = down_label;
	browser->dir_label      = dir_label;
	browser->slidebar_label = slidebar_label;
	browser->tot_suffix     = 0;
	browser->page_color       = page_color;
	browser->background_color = background_color;
	
	LUI_Browser = browser;
    }
    else {
	LUI_Browser->x           = x;
	LUI_Browser->y           = y;
	LUI_Browser->width       = width;
	LUI_Browser->height      = height;
	LUI_Browser->func        = func;
    }

    /* Prepare to create directory */
    LUI_Browser->pixmap_yoffset = 0;

    return(LUI_Browser);
}

/******************************************************************************/
 static
LUI_BrowserDispatch(button)
    LUI_BUTTON *button;
{
    char str[2048], *LUI_BrowserCurrentDirectory();
    int LUI_BrowserSetDirectory();
    static int been_here = 0;

    if (!(strcmp(button->label, "Change Dir"))) {
	if (!been_here) {
	    int x, y, w, h;

	    been_here = 1;

	    h = 260 - (2 * 20);
	    w = 700;
	    x = 260;
	    y = 1024 - h;
	    LUIdialog_create("LUI_BrowserDialog", LUI_DIALOG_INPUT, LUI_RootWindow, 
			     x, y, w, h, 
			     LUI_Browser->page_color,
			     LUI_Browser->background_color);
	}
	
	sprintf(str, "   %s", LUI_BrowserCurrentDirectory());
	LUIdialog_request("LUI_BrowserDialog", 
			  "Enter name of directory to change to.  Currently in:", 
			  str, LUI_BrowserSetDirectory);
    }
    else if (!(strcmp(button->label, "Close "))) 
      LUI_BrowserVisible(0);
}

/******************************************************************************/
LUI_BrowserDefineSuffixList(n, suffix_list)
    int n;
    char *suffix_list[];
{
    int i;

    LUI_Browser->tot_suffix = n;
    
    for (i = 0; i < n; ++i) 
      strcpy(LUI_Browser->suffix_list[i], suffix_list[i]);
}

/******************************************************************************/
 char *
LUI_BrowserCurrentFilename()
{
    return(LUI_BrowserFilename);
}

/******************************************************************************/
 char *
LUI_BrowserCurrentDirectory()
{
    return(LUI_Browser->directory);
}

/******************************************************************************/
LUI_BrowserVisible(on)
    int on;
{
    if (on) {
	XMapRaised(LUI_Display, LUI_Browser->big_window);
	XMapWindow(LUI_Display, LUI_Browser->window);
	XMapWindow(LUI_Display, LUI_Browser->browser_window);
	LUI_Browser->visible = 1;
    }
    else {
	XUnmapWindow(LUI_Display, LUI_Browser->big_window);
/* REMOVED
	XUnmapWindow(LUI_Display, LUI_Browser->window);
	XUnmapWindow(LUI_Display, LUI_Browser->browser_window);
*/
	LUI_Browser->visible = 0;
    }

    return(1);
}

/******************************************************************************/
LUI_BrowserDraw()
{
    /* If not visible, don't draw it */
    if (!LUI_Browser->visible) 
      return;

    LUI_BrowserDrawText(NULL);
}

/******************************************************************************/
LUI_BrowserDrawText(event)
    XEvent *event;
{
    int x, y, xs, ys, ls;
    static int ind = -1, old_ind = -1;

    if (!event) {
	LUI_BrowserCopyTextPixmap();
	old_ind = -1;
    }

    else {
	ls  = LUI_Browser->line_space;
	ind = (event->xbutton.y + LUI_Browser->pixmap_yoffset) / ls;
	x   = 0;
	y   = (event->xbutton.y / ls) * ls;
	xs  = LUI_Browser->browser_width - 1;
	ys  = (LUI_Browser->font_yoff + LUI_BROWSER_LINE_SPACE);

	if (ind >= LUI_Browser->tot_files) {
	    LUI_BrowserCopyTextPixmap();
	    return;
	}

	if (ind == old_ind)
	  return;
	old_ind = ind;
	LUI_Browser->file_ind = ind;

	LUI_BrowserCopyTextPixmap();

	LUI_SetColor(LUI_Color_white);
	XFillRectangle(LUI_Display, LUI_Browser->browser_window, LUI_Gc, x, y, xs, ys);
	
	LUI_BrowserDrawString(LUI_Browser->browser_window, ind, 
			      y + ys - LUI_BROWSER_LINE_SPACE/2, 1);
    }
}

/******************************************************************************/
 static
LUI_BrowserCopyTextPixmap()
{
    XCopyArea(LUI_Display, 
	      LUI_Browser->pixmap, LUI_Browser->browser_window,
	      LUI_Gc,
	      0, LUI_Browser->pixmap_yoffset,
	      LUI_Browser->width, LUI_Browser->height,
	      0, 0);
}

/******************************************************************************/
 static
LUI_BrowserLabelProcess(label)
    LUI_LABEL *label;
{
    int i, way, offset;
    static int pressed = 0;
    float secs;

    if (label == LUI_Browser->slidebar_label) 
      way = (LUI_Browser->browser_height / 
	    (LUI_Browser->font_yoff + LUI_BROWSER_LINE_SPACE));
    else 
      way = 1;

    switch (label->event->type) {
      case EnterNotify:
	LUI_LabelHighlight(label->name, 1);
	break;
      case LeaveNotify:
	if (!pressed)
	  LUI_LabelHighlight(label->name, 0);
	break;
      case ButtonPress:
	if (label->event->xbutton.button == Button1) 
	  way *= -1;

	if (way < 0) 
	  LUI_Cursor(label->window, LUI_UP_CURSOR);
	else
	  LUI_Cursor(label->window, LUI_DOWN_CURSOR);

	/* Set the timer */
	LUI_Timer(31, 0, &secs);
	
	for (pressed = 1; pressed; ) {
	    /* Drop out of loop if button is released */
	    while (XPending(LUI_Display)) {
		XEvent event;
		
		XNextEvent(LUI_Display, &event);
		LUI_EventDispatch(&event);
	    }
	    if (!pressed) 
	      break;

	    /* Compute browser window offset */
	    offset = (way * (LUI_Browser->font_yoff + LUI_BROWSER_LINE_SPACE));

	    LUI_Browser->pixmap_yoffset += offset;
	    
	    if (LUI_Browser->pixmap_yoffset < 0) {
		LUI_Browser->pixmap_yoffset = 0;
		offset = 0;
	    }
	    else if ((LUI_Browser->pixmap_yoffset /* + LUI_Browser->browser_height */) > 
		     LUI_Browser->pixmap_height) {
		LUI_Browser->pixmap_yoffset -= offset;
		offset = 0;
	    }

	    /* Draw the new browser window */
	    LUI_BrowserDrawText(NULL);
	    XSync(LUI_Display, 0);
	    
	    /* Time out */
	    for (;;) {

		LUI_Timer(31, 1, &secs);
		if (secs > 0.35) 
		  break;

		/* Drop out of loop if button is released */
		while (XPending(LUI_Display)) {
		    XEvent event;
		    
		    XNextEvent(LUI_Display, &event);
		    LUI_EventDispatch(&event);
		}
		if (!pressed) 
		  break;
	    }
	    XSync(LUI_Display, 0);
	}
	break;
      case ButtonRelease:
	LUI_Cursor(label->window, LUI_UP_DOWN_CURSOR);
	pressed = 0;
	break;
    }
}

/******************************************************************************/
 static
LUI_BrowserProcess(window, event)
    Window window;
    XEvent *event;
{
    int ind = LUI_Browser->file_ind;

    /* Process the event */
    switch (event->type) {
      case ButtonPress:
	/* Selected a DIRECTORY */
	if (LUI_Browser->file_list[ind].type == LUI_FILE_DIRECTORY) 
	  LUI_BrowserSetDirectory(LUI_Browser->file_list[ind].filename);

	/* Selected a FILE */
	else if (LUI_Browser->func) {
	    if ((strlen(LUI_Browser->directory) == 1) && *(LUI_Browser->directory) == '/') 
	      sprintf(LUI_BrowserFilename, "/%s", LUI_Browser->file_list[ind].filename);
	    else 
	      sprintf(LUI_BrowserFilename, "%s/%s", 
		      LUI_Browser->directory, LUI_Browser->file_list[ind].filename);
	    
	    (*(LUI_Browser->func))(LUI_BrowserFilename);
	}

	break;
      case ButtonRelease:
	break;
      case EnterNotify:
	break;
      case LeaveNotify:
	LUI_BrowserDrawText(NULL);
	break;
      case MotionNotify:
	LUI_BrowserDrawText(event);
	break;
      case Expose:
	LUI_BrowserDrawText(NULL);
	break;
    }
    return(1);
}

/******************************************************************************/
 static
LUI_BrowserDirLabel()
{
    char *s;
    LUI_LABEL *label = LUI_Browser->dir_label;

    /* Create the current directory name */
    s = PathTail(LUI_Browser->directory);

    /* Copy in new name and redraw label */
    if (*s == '\0')
      strcpy(LUI_Browser->dir_label_text, "/");
    else 
      strcpy(LUI_Browser->dir_label_text, s);

    label->argv[0] = LUI_Browser->dir_label_text;
    
    LUI_LabelDrawText(label);
    
    return(1);
}


/******************************************************************************/
 static char *
PathTail(buf)
    char *buf;
{
  char **strs;
  int n;

  PathParse(buf, &n, &strs);

  if (n) 
    return(strs[n-1]);
  else 
    return("");
}
    

/******************************************************************************/
 static char *
PathTailUp(buf)
    char *buf;
{
  char **strs;
  int n;

  PathParse(buf, &n, &strs);

  if (n > 1) 
    return(strs[n-2]);
  else 
    return("");
}
    
/******************************************************************************/
 static
PathParse(path, n, strs)
    char *path;
    int *n;
    char ***strs;
{
    static char *files[64], tmp_path[LUI_FILENAME_SIZE];
    char *c, *s, *strchr();
    int ind = 0;

    strcpy(tmp_path, path);

    for (s = tmp_path; c = strchr(s, '/'); s = c + 1) {
	*c = '\0';
	if (strlen(s))
	  files[ind++] = s;
    }
    files[ind++] = s;

    *n = ind;
    *strs = files;
}
    
/******************************************************************************/
 static
LUI_BrowserWindowProcess(window, event)
    Window window;
    XEvent *event;
{
    switch (event->type) {
      case Expose:
	if (event->xexpose.count > 0) 
	  return;

	if (LUI_Browser->title_pixmap)
	  XCopyArea(LUI_Display, LUI_Browser->title_pixmap, LUI_Browser->big_window,
		    LUI_GC_fore, 0, 0, LUI_Browser->title_width, LUI_Browser->title_height,
		    10, 8);

	LUI_DrawDropShadow(LUI_Browser->big_window, 
			   LUI_Browser->x, LUI_Browser->y,
			   LUI_Browser->width, LUI_Browser->height,
			   6, LUI_Color_black);

	LUI_DrawFlatBorderAt(LUI_Browser->big_window, 
			     0, 0, LUI_Browser->big_width, LUI_Browser->big_height, 
			     4, LUI_MakeColor(10, 10, 10));
	LUI_BrowserDraw();
	break;
    }
}

/******************************************************************************/
LUI_BrowserMakePixmap()
{
    register int i, y;
    static int been_here = 0;

    if (been_here) {
	/* Destroy the pixmap */
	XFreePixmap(LUI_Display, LUI_Browser->pixmap);
    }
    else 
      been_here = 1;

    LUI_Browser->pixmap_height = LUI_Browser->tot_files * LUI_Browser->line_space;

    /* Create a pixmap */
    LUI_Browser->pixmap = XCreatePixmap(LUI_Display, 
			   LUI_RootWindow, 
			   LUI_Browser->width, 
                           LUI_Browser->pixmap_height + LUI_Browser->browser_height,
			   LUI_Depth);

    LUI_SetColor(LUI_MakeColor(120, 120, 120));
    XFillRectangle(LUI_Display, LUI_Browser->pixmap, LUI_Gc, 
		   0, 0, LUI_Browser->width,
		   LUI_Browser->pixmap_height + LUI_Browser->browser_height);

    /* Draw the file names into the pixmap */
    for (i = 0; i < LUI_Browser->tot_files; ++i) {
	y = i * (LUI_Browser->line_space) + 
	  LUI_Browser->font_yoff + LUI_BROWSER_LINE_SPACE / 2;

	LUI_BrowserDrawString(LUI_Browser->pixmap, i, y, 0);
    }
}

/******************************************************************************/
 static
LUI_BrowserDrawString(xid, ind, y, hilite)
    XID xid;
    int ind, y, hilite;
{
    int size;

    size = strlen(LUI_Browser->file_list[ind].filename);
    if (!size) 
      return;
    
    if (LUI_Browser->file_list[ind].type == LUI_FILE_DIRECTORY) 
      LUI_SetColor(LUI_MakeColor(200, 0, 0));
    else if (LUI_Browser->file_list[ind].type == LUI_FILE_GEOM) 
      LUI_SetColor(LUI_MakeColor(0, 200, 0));
    else if (LUI_Browser->file_list[ind].type == LUI_FILE_POBJ) 
      LUI_SetColor(LUI_MakeColor(0, 0, 200));
    else 
      LUI_SetColor(LUI_MakeColor(0, 0, 0));

    XDrawString(LUI_Display, xid, LUI_Gc, 20, y, LUI_Browser->file_list[ind].filename, size);
}


/******************************************************************************/
LUI_FileCompare(f1, f2)
    LUI_DIR_ENT *f1, *f2;
{
    return(strcmp(f1->filename, f2->filename));
}


/******************************************************************************/
 static
LUI_FileMakeList(name)
    char *name;
{ 
    DIR *dirf;
    struct dirent *dentry;
    char path_name[LUI_FILENAME_SIZE], file_name[LUI_FILENAME_SIZE], *s;
    int type;

    if(!(dirf = opendir(name))) {
	LUI_Error("LUI_FileMakeList(): can't open directory %s\n", name);
	return(0);
    }

    for (LUI_Browser->tot_files = 0; dentry = readdir(dirf); ) {
	strcpy(file_name, dentry->d_name);

	/* We don't want home directory to show up */
	if (!(strcmp(file_name, "."))) 
	  continue;

	/* Special treatment for up directory */
	else if (!(strcmp(file_name, ".."))) {
	    if ((strlen(name) == 1) && *name == '/')
	      continue;
	    s = PathTailUp(name);
	    sprintf(file_name, "../%s", s);
	    LUI_Browser->file_list[LUI_Browser->tot_files].type = LUI_FILE_DIRECTORY;
	}

	/* Regular file or directory */
	else {
	    /* Directory */
	    if ((strlen(name) == 1) && *name == '/')
	      sprintf(path_name, "/%s", file_name);

	    else 
	      sprintf(path_name, "%s/%s", name, file_name);

	    if ((type = LUI_FileDescribe(path_name)) != LUI_FILE_DIRECTORY) {
		/* File: Check for valid suffix */
		if (LUI_Browser->tot_suffix) {
		    int i;
		    
		    /* Loop through suffixes */
		    for (i = 0; i < LUI_Browser->tot_suffix; ++i) {
			if (str_suffix(file_name, LUI_Browser->suffix_list[i])) {
			    sprintf(path_name, "%s/%s", name, file_name);
			    break;
			}
		    }
		    
		    /* Couldn't find a valid suffix */
		    if (i == LUI_Browser->tot_suffix) 
		      continue;
		}
	    }
	    LUI_Browser->file_list[LUI_Browser->tot_files].type = type;
	}

	/* Allocate name and copy it in */
	if (!(LUI_Browser->file_list[LUI_Browser->tot_files].filename = 
	      (char *)malloc(strlen(file_name) + 1))) {
	    fprintf(stderr, "LUI_FileMakeList(): can't malloc filename\n");
	    return(0);
	}
	strcpy(LUI_Browser->file_list[LUI_Browser->tot_files].filename, file_name);
	++LUI_Browser->tot_files;

    }
    closedir(dirf);

    qsort((char *)LUI_Browser->file_list, LUI_Browser->tot_files, sizeof(LUI_DIR_ENT), LUI_FileCompare);

    return(1);
}


/******************************************************************************/
LUI_FileFreeList()
{
    register int i;

    for (i = 0; i < LUI_Browser->tot_files; ++i) {
	free(LUI_Browser->file_list[i].filename);
    }
}

/******************************************************************************/
LUI_FileDescribe(name)
    char *name;
{ 
    int type;
    struct stat	stat_buff;

    stat(name,&stat_buff);

    /* Can't walk links */
    if(stat_buff.st_mode & S_IFLNK)
      type = LUI_FILE_UNKNOWN;

    /* Directory */
    else if(stat_buff.st_mode & S_IFDIR)
      type = LUI_FILE_DIRECTORY;

    /* Character special */
    else if(stat_buff.st_mode & S_IFCHR)
      type = LUI_FILE_UNKNOWN;

    /* Block special */
    else if(stat_buff.st_mode & S_IFBLK)
      type = LUI_FILE_UNKNOWN;

#ifdef FOO
    else if(stat_buff.st_mode & S_IFREG) {
	int fd;
	unsigned short magic;
	unsigned char magic1, magic2;
	char fbuf[BUFSIZ];
	int i;
	
	fd = open(name,O_RDONLY);
	read(fd,&magic,sizeof(magic));

	switch(magic) {
          /* Stellar executable */
	  case 0x180: 
	    type = LUI_FILE_UNKNOWN;
	    break;

	  /* Archive */
	  case 0x213c:
	    type = LUI_FILE_UNKNOWN;
	    break;

	  /* Shell script */
	  case 0x2321:
	    type = LUI_FILE_UNKNOWN;
	    break;

	  default:
	    read(fd,fbuf,BUFSIZ);
	    magic1 = *fbuf;
	    magic2 = *(fbuf + 1);

	    /* GEOM file */
	    if (magic == 0x4745 && magic1 == 0x4f && magic2 == 0x4d)
	      type = LUI_FILE_GEOM;
	    
	    /* POBJ file */
	    else if (magic == 0x00 && magic1 == 0x0b && magic2 == 0x97)
	      type = LUI_FILE_POBJ;
	    else if (magic == 0x00 && magic1 == 0x0b && magic2 == 0x98)
	      type = LUI_FILE_POBJ;
	    else if (magic == 0x00 && magic1 == 0x0b && magic2 == 0x99)
	      type = LUI_FILE_POBJ;
	    
	    else {
		i = 0;
		while(i<256 && fbuf[i] != '\n')
		  i++;
		/* Data */
		if(i==256) 
		  type = LUI_FILE_UNKNOWN;
		
		/* Text */
		else type = LUI_FILE_UNKNOWN;
	    }
	}
	close(fd);
    }
#endif

    /* Fifo */
    else if(stat_buff.st_mode & S_IFIFO)
      type = LUI_FILE_UNKNOWN;

    /* Socket */
    else if(stat_buff.st_mode & S_IFSOCK)
      type = LUI_FILE_UNKNOWN;

    /* Unknown */
    else
      type = LUI_FILE_UNKNOWN;

   return(type);
}

/******************************************************************************/
LUI_BrowserLabelDrawArrow(label, x, y, width, height)
    LUI_LABEL *label;
    int x, y, width, height;
{
    static XPoint up[3], dn[3];
    static int been_here;

    if (!been_here) {
	been_here = 1;

	up[0].x = 0;
	up[1].x = width * 0.30;
	up[2].x = width * 0.60;
	
	dn[0].x = width - (width * 0.30);
	dn[1].x = width - (width * 0.60);
	dn[2].x = width;
	
	up[0].y = height * 0.75;
	up[1].y = height * 0.25;
	up[2].y = up[0].y;
	
	dn[0].y = up[2].y;
	dn[1].y = up[1].y;
	dn[2].y = up[1].y;
    }	
    XFillPolygon(LUI_Display, label->window, 
		 LUI_GC_dk_grey, up, 3, Nonconvex, CoordModeOrigin);
    XFillPolygon(LUI_Display, label->window, 
		 LUI_GC_dk_grey, dn, 3, Nonconvex, CoordModeOrigin);
}

/******************************************************************************/
LUI_BrowserSetDirectory(dir)
    char *dir;
{
    int ok;
    char new_dir[LUI_FILENAME_SIZE], *c;

    /* Up directory */
    if (!(strcmp(dir, "../"))) {
	strcpy(new_dir, "/");
    }
    else if (!(memcmp(dir, "..", 2)) || !(memcmp(dir, "../", 3))) {
	strcpy(new_dir, LUI_Browser->directory);
	
	/* Walk backwards from last character */
	for (c = new_dir + strlen(new_dir) - 1; *c != '/'; --c);
	
	/* If not at root, null the last slash */
	if (c != new_dir) 
	  *c = '\0';
    }

    /* Absolute path */
    else if (*dir == '/') 
      strcpy(new_dir, dir);
    
    /* Relative path */
    else {
	if ((strlen(LUI_Browser->directory) == 1) && *LUI_Browser->directory == '/')
	  sprintf(new_dir, "/%s", dir);
	else
	  sprintf(new_dir, "%s/%s", LUI_Browser->directory, dir);
    }
    
    /* Null the last slash */
    if (strlen(new_dir) != 1 && *new_dir != '/') {
	c = new_dir + strlen(new_dir) - 1;
	if (*c == '/')
	  *c = '\0';
    }

    if (LUI_Debug)
      fprintf(stderr, "LUI_BrowserSetDirectory(): in (%s) changing to (%s)\n", 
	      LUI_Browser->directory, new_dir);
    
    /* Create the list of files */
    XGrabPointer(LUI_Display,
		 LUI_RootWindow,
		 True,
		 ButtonPressMask 
		 | ButtonReleaseMask 
		 | ButtonMotionMask 
		 | LeaveWindowMask
		 | EnterWindowMask,
		 GrabModeAsync,
		 GrabModeAsync,
		 None,
		 XCreateFontCursor(LUI_Display,XC_watch),
		 CurrentTime);
    XSync(LUI_Display, 0);
    
    if (LUI_FileMakeList(new_dir)) {
	strcpy(LUI_Browser->directory, new_dir);
	
	/* Redraw directory label */
	LUI_BrowserDirLabel();
	
	/* Reset pixmap offset */
	LUI_Browser->pixmap_yoffset = 0;
	
	/* Create the pixmap */
	LUI_BrowserMakePixmap();
	
	/* Copy a piece of the pixmap into the window */
	LUI_BrowserCopyTextPixmap();
	LUI_BrowserDrawText(NULL);
	
	ok = 1;
    }
    else
      ok = 0;

    XUngrabPointer(LUI_Display, CurrentTime);
    XSync(LUI_Display, 0);

    return(ok);
}

/******************************************************************************/
 static
str_suffix(str, suffix)
    char *str, *suffix;
{
    char *c;
    int len;

    len = strlen(suffix);

    c = str + strlen(str) - len;

    return(!memcmp(c, suffix, len));
}
